import re
import sys

from os import makedirs
from os.path import join, exists

from argparse import ArgumentParser
from jinja2 import PackageLoader, Environment, FileSystemLoader

from .helper import ArgparseHelper


class Command(object):
    """
    项目构建工具
    """
    def __init__(self):
        args = self.parse_args()
        self.templates_path = args.templates
        self.names = args.name
        self.type = args.type

    def create(self):
        if self.templates_path:
            env = Environment(loader=FileSystemLoader(self.templates_path))
        else:
            env = Environment(loader=PackageLoader('star_builder', 'templates'))
        getattr(self, f"create_{self.type}")(env)

    def create_project(self, env):
        for name in self.names:
            makedirs(name, exist_ok=True)
            makedirs(join(name, "static"), exist_ok=True)
            makedirs(join(name, "templates"), exist_ok=True)
            template = env.get_template(join(self.type, 'start.py.tmpl'))
            with open(join(name, "start.py"), "w") as f:
                f.write(template.render())
            print(f"{name} 项目已完成创建。")

    def create_service(self, env):
        father = None
        if exists("__init__.py"):
            regex = re.compile(r"class\s+(\w*?Service)\(\w*Service\):")
            mth = regex.search(open("__init__.py").read())
            if mth:
                father = mth.group(1)

        for name in self.names:
            makedirs(name, exist_ok=True)
            template = env.get_template(join(self.type, '__init__.py.tmpl'))
            with open(join(name, "__init__.py"), "w") as f:
                f.write(template.render(father=father or "Service", service=name))

        print("、".join(self.names), "服务模块已完成创建。")

    def parse_args(self):
        base_parser = ArgumentParser(
            description=self.__class__.__doc__, add_help=False)
        base_parser.add_argument("-t", "--templates", help="模板路径.")

        parser = ArgumentParser(description="Apistar项目构建工具", add_help=False)
        parser.add_argument('-h', '--help', action=ArgparseHelper,
                            help='显示帮助信息并退出. ')
        sub_parsers = parser.add_subparsers(dest="type", help="创建项目还是服务模块 ")

        project = sub_parsers.add_parser(
            "project", parents=[base_parser], help="创建项目")
        project.add_argument("name", nargs=1, help="项目名称")

        service = sub_parsers.add_parser(
            "service", parents=[base_parser], help="创建服务模块")
        service.add_argument("name", nargs="+", help="服务模块名称")
        if len(sys.argv) < 2:
            parser.print_help()
            exit(1)
        return parser.parse_args()


def main():
    Command().create()


if __name__ == "__main__":
    main()



import logging

from apistar import ASyncApp

from .helper import load_packages, routing
from .service import Service

__all__ = ["Application"]

logger = logging.getLogger("app")


class Application(ASyncApp):
    """
    可以动态发现当前项目根目录下所有routes的service中的handler
    """
    def __init__(self,
                 template_dir=None,
                 static_dir=None,
                 schema_url='/schema/',
                 static_url='/static/',
                 components=None,
                 event_hooks=None):
        load_packages(".")
        include = routing(Service, None)
        if include:
            routes = [include]
        else:
            logger.info("Noting to route. ")
            routes = []
        super(Application, self).__init__(
            routes,
            template_dir,
            static_dir,
            schema_url,
            static_url,
            components,
            event_hooks)



# -*- coding:utf-8 -*-
from apistar import Component, Route
from .route import route


class ServiceMeta(type):
    """
    元类的主要作用是建立Service继承树，以便发现所有Service。
    """
    def __new__(mcs, class_name, bases, props):
        cls = super(ServiceMeta, mcs).__new__(mcs, class_name, bases, props)
        cls.children = []

        if len(bases) > 1:
            raise RuntimeError("Service class unsupprt multi inherit!")

        if not bases or bases[0] == object:
            return cls

        bases[0].children.append(cls)
        return cls


@route("")
class Service(metaclass=ServiceMeta):
    """
    所有一级模块业务类都需要继承于Service类
    """
    pass


class ServiceComponent(Component):
    """
    依赖注入Service组件
    """
    def resolve(self, route: Route) -> Service:
        return route.service




# -*- coding:utf-8 -*-
from apistar import Route


__all__ = ["route", "get", "post", "delete", "put", "options"]


def route(prefix, name=None):
    """
    route为service指定名称和路径前缀
    :param prefix:
    :param name:
    :return:
    """
    def prefix_wrapper(cls):
        cls.prefix = prefix
        cls.name = name or cls.__name__.lower()
        return cls

    return prefix_wrapper


def wrapper_method(method_name):
    """
    生成绑定http method的route方法
    @get()
    async def hello():
        return {"message": "hello world"}
    :param method_name: get
    :return:
    """
    def method(url=None, name=None, documented=True, standalone=False, link=None):

        def endpoint_wrapper(handler):
            handler.route = Route(
                url or "/" + handler.__name__, method_name.upper(), handler, name,
                documented, standalone, link)
            return handler

        return endpoint_wrapper

    method.__name__ = method_name
    method.__qualname__ = method_name
    return method


get = wrapper_method("get")
post = wrapper_method("post")
delete = wrapper_method("delete")
options = wrapper_method("options")
put = wrapper_method("put")



# -*- coding:utf-8 -*-
from .builder import main
from .app import Application
from .service import Service, ServiceComponent
from .route import route, get, post, delete, put, options


__version__ = "0.0.5"
